#######################################################################
#
# NCM component for filecopy daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 1.0.6 : 21/12/04 14:07
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::filecopy;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;

use File::Basename;
use File::Path;

local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define path for convenience. 
    my $base = "/software/components/filecopy";

    # Determine first if there is anything to do.
    return 0 unless ($config->elementExists("$base/services"));

    # Extract the list of files to manage. 
    my %files = $config->getElement("$base/services")->getHash();

    # Loop over all of the files and manage the associated services.
    my @commands;
    for my $encfname (sort keys %files) {

	# Convenience variables. 
	my $configPath = "$base/services/$encfname/config";
	my $restartPath = "$base/services/$encfname/restart";
	my $permsPath = "$base/services/$encfname/perms";
	my $ownerPath = "$base/services/$encfname/owner";

	# The actual file name.
	my $fname = unescape($encfname); 

	# Pull in the configuration.
	my $contents = $config->getValue($configPath);

	# See if the old configuration is the same. 
	my $ocontents = '';
	if (-e $fname) {
	    open FILE, "<", "$fname";
	    $ocontents = <FILE>;
	    close FILE;
	}

	# Get the permissions if specified.  Set to -1 if not. 
	my $perms = -1;
	if ($config->elementExists($permsPath)) {
	    $perms = oct($config->getValue($permsPath));
	}

	# Check to see if the permissions are different.
	my $operms = -1;
	if ($perms >= 0) {
	    $operms = (stat($fname))[2] if (-e $fname);
	}

	# If the permissions or contents have changed, do something.
	my $changed = (($perms >= 0) && ($perms != $operms)) ||
	    ($contents ne $ocontents);

	# Only do something if necessary.
	if ($changed) {

	    # Now just create the new configuration file.  Be careful to save
	    # a backup of the previous file if necessary. 
	    if ( ! -e $fname ) {

		# Check to see if the directory needs to be created.
		my $dir = dirname($fname);
		mkpath($dir,0,0755) unless (-e $dir);
		if (! -d $dir) {
		    $self->error("Can't create directory: $dir");
		    next;
		}
		
		# Configuration file doesn't exist yet.  Create it. 
		open ( CONF,">$fname" );
		print CONF $contents;
		close (CONF);
		$self->log("$fname created");
		
	    } else {
		
		# Already exists. Make backup and create new file. 
		my $result = LC::Check::file( $fname,
					      backup => ".old",
					      contents => $contents,
					      );
		$self->log("$fname updated") if $result;
	    }
	}

	# Change the permissions if given.
	chmod $perms, $fname if ($perms >= 0);

	# Set the owner (default group) if given.
	if ($config->elementExists($ownerPath)) {
	    my $owner = $config->getValue($ownerPath);
	    chown((getpwnam($owner))[2,3], ($fname));
	}

	# Queue the restart command if given. 
	if ($config->elementExists($restartPath)) {
	    my $cmd = $config->getValue($restartPath);
	    push @commands, $cmd;
	}

    }

    # Loop over all of the commands and execute them.  Do this after
    # everything to take care of any dependencies between writing
    # multiple files.
    foreach (@commands) {
	my $rc = system($_);
	$self->error("Failed restart: $_\n") if ($rc);
    }

    return 1;
}


#
# small helper function for unescaping chars
# (taken from the spma component)
#
sub unescape ($) {
  my $str=shift;

  $str =~ s!(_[0-9a-f]{2})!sprintf("%c",hex($1))!eg;
  return $str;
}


1;      # Required for PERL modules
